package cz.cvut.fit.swing.calipel;

import cz.cvut.fit.swing.calipel.core.BenchmarkRunner;
import cz.cvut.fit.swing.calipel.core.BenchmarkSuite;
import cz.cvut.fit.swing.calipel.core.Configuration;
import cz.cvut.fit.swing.calipel.printer.JsonPrinter;
import cz.cvut.fit.swing.calipel.printer.Printer;
import cz.cvut.fit.swing.calipel.printer.StdOutPrinter;
import org.apache.commons.cli.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


public class Main {

    public static void main(String[] args) {
        Options options = describeAllowedOptions();
        try {
            CommandLine cmd = parseCmdLineOptions(args, options);
            processCmdLineOptions(options, cmd);
        } catch (ParseException e) {
            System.err.println("Parsing failed.  Reason: " + e.getMessage());
            printHelp(options);
        }
    }

    private static Options describeAllowedOptions() {
        Options options = new Options();

        options.addOption("h", "help", false, "print this message");
        options.addOption("b", "benchmarks", true, "specify which benchmarks to run");
        options.addOption("j", "json", false, "use json printer");
        return options;
    }

    private static CommandLine parseCmdLineOptions(String[] args,
                                                   Options options) throws ParseException {
        CommandLineParser parser = new GnuParser();
        return parser.parse(options, args);
    }

    private static void processCmdLineOptions(Options options, CommandLine cmd) {
        Printer printer = new StdOutPrinter();
        Configuration configuration = new Configuration();
        List<String> benchmarksToRun = Arrays.asList("DeltaBlue", "BenchmarkMicro");

        if (cmd.hasOption("h")) {
            printHelp(options);
        } else if (cmd.hasOption("n")) {
            configuration.setNumOfTries(Integer.parseInt(cmd.getOptionValue("n")));
        } else if (cmd.hasOption("j")) {
            printer = new JsonPrinter();
        } else if (cmd.hasOption("b")) {
            benchmarksToRun = Arrays.asList(cmd.getOptionValue("b").split(","));
        }

        BenchmarkSuite benchmarks = new BenchmarkSuite(benchmarksToRun, configuration);
        BenchmarkRunner runner = new BenchmarkRunner(benchmarks, printer, configuration);
        runner.start();
    }

    private static void printHelp(Options options) {
        HelpFormatter formatter = new HelpFormatter();
        formatter.printHelp("calipel-java <args>", options);
    }
}
