package cz.cvut.fit.swing.calipel.printer;

import cz.cvut.fit.swing.calipel.core.BenchmarkOutcome;
import cz.cvut.fit.swing.calipel.core.BenchmarkResult;
import org.json.simple.JSONObject;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

public class JsonPrinter extends Printer {

    @Override
    public void printResult(BenchmarkResult result) {
        JSONObject json = new JSONObject();
        List<Map> outcomes = new ArrayList<Map>();
        for (BenchmarkOutcome measurement : result.getMeasuredBenchmarks()) {
            outcomes.add(measurement.toMap());
        }

        String hostname = findHostname();
        String timestamp = getCurrentTimestampInISOFormat();

        Map<String, String> configuration = new HashMap<String, String>();
        configuration.put("language", "Java");
        configuration.put("runtime", System.getProperty("java.runtime.version"));
        configuration.put("os", System.getProperty("os.name"));
        configuration.put("machineid", hostname);

        json.put("tags", "default");
        json.put("timestamp", timestamp);
        json.put("configuration", configuration);
        json.put("outcomes", outcomes);

        System.out.println(json.toJSONString());
    }

    private String findHostname() {
        String hostname = System.getenv("COMPUTERNAME");
        if (hostname == null) {
            hostname = System.getenv("HOSTNAME");
        }

        if (hostname == null) {
            try {
                hostname = InetAddress.getLocalHost().getHostName();
            } catch (UnknownHostException e) {
                hostname = null;
            }
        }

        if (hostname == null) {
            throw new RuntimeException("C'mon don't make me implement something clever " +
                                             "here and provide me env vars");
        }

        return hostname;
    }

    private String getCurrentTimestampInISOFormat() {
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mmZ");
        return df.format(new Date());
    }
}